//This file is create_vin.cc 
//Definition of member function of Lattice class
//The func. transfers interactions to Vin vectors in class Ncluster 
//and print to file the Vin
//By Teck Leong Tan
#include"lattice.h"

void Lattice::create_Vin(const vector<double>& J)
{
  fstream potfile;
  potfile.open("in_pot.txt", ios::out);
  if (!potfile)
  {
    cout<<"File: "<<"in_pot_sinv.txt"<<" is not opened!"<<endl;
    exit(1);
  }
  potfile<<"#isublattice, ibody, icluster_Ds, Vin"<<endl;

  //Insert constant term of interaction in Vin
  clustree.sublat[0].nbody[0].nclus[0]->Vin[0]=J[0];
  potfile<<"0 0 0 "<<J[0]<<endl;

  //Insert rest of cluster interactions
  //Loop over no. of sublattice
  for (int l=0; l<num_sublat;l++)
  {
    //Loop over no. of atoms in cluster
    for (int n=1;n<maxbody+1;n++)
    {
      //Loop over cluster types
      for (int f=0;f<clustree.sublat[l].nbody[n].nclus.size();f++)
      {
        potfile<<l<<" "<<n<<" "<<f<<" ";
        for (int aa=0; aa<power(num_type,n);aa++)
        {
          int index;
	  int findex=clustree.sublat[l].nbody[n].nclus[f]->f_index;//Real f-index
          index=clustree.sublat[0].nbody[n].all_clus[findex][aa];
          clustree.sublat[l].nbody[n].nclus[f]->Vin[aa]=J[index];
          potfile<<J[index]<<" ";
        }
        potfile<<endl;
      }
    }
  }//END for (int l=0; l<num_sublat;l++)

  //Some checking
  //If zero body Vin is not defined, set it to zero.
  if (clustree.sublat[0].nbody[0].nclus[0]->Vin.empty())
  {
    //clustree.sublat[0].nbody[0].nclus[0]->Vin.push_back(0);
    cout<<"Error in create_vin.cc: Constant term interaction should not be empty"<<endl;
    exit(1);
  }
 
  if (clustree.sublat[0].nbody[maxbody].nclus[0]->Vin.empty())
  {
    cout<<"Error in create_Vin.cc: No. of potentials given does not match no.";
    cout<<"of interacting bodies in input file"<<endl;
  }

  //====================================================================
  //The following ensures that all sublattices contains interaction 
  //potentials, by looking at the size of ...sublat[]...Vin[1].
  //If size=0, assume symmetery and copy ...sublat[0]...Vin[] to 
  //...sublat[empty Vin]...Vin[];
  //===================================================================
  //Loop over sublattice
  for (int l=1; l<num_sublat; l++)
  {
    if (clustree.sublat[l].nbody[1].nclus[0]->Vin.size() ==0)
    {
      //Loop over nbody
      for (int n=0; n<maxbody+1;n++)
      {
        for (int f=0; f<clustree.sublat[l].nbody[n].nclus.size(); f++)
        {
          //Check that the size of ...Vin is zero;
          if (clustree.sublat[l].nbody[n].nclus[f]->Vin.size()>0)
          {
            cout<<"create_Vin.cc: Error in Vin"<<endl;
            exit(1);
          }  
          for (int i=0; i<clustree.sublat[0].nbody[n].nclus[f]->Vin.size();i++)
          {
            double temp;
            temp=clustree.sublat[0].nbody[n].nclus[f]->Vin[i];
            clustree.sublat[l].nbody[n].nclus[f]->Vin.push_back(temp);
          }
        }
      }//end for (int n=0; n<maxbody;n++)
    }
  }//end for (int l=1; l<num_sublat; l++)  
}
//EOF
