//This file is "lattice.h"
//Header file for class Lattice
//By Teck Leong Tan

#ifndef _LATTICE_H
#define _LATTICE_H
#define  MAXCHAR 30
#include <iostream>
#include <fstream>
#include <math.h>
#include <vector>
#include <iomanip>
#include <string>
#include "energy.h"
#include "concentration.h"
#include "clustertree.h"
#include "sublattice.h"
#include "nbody.h"
#include "power.h"
#include "cello.h"
#include "siteo.h"
#include "filenames.h"
#include "cor_def_param.h"
#include "keywords.h"
#include "inversion.h"
#include "nei_pos.h"
#include "element.h"
#include "trans_vectors.h"

using namespace std;

class Lattice
{
  public:
    //Member data-----------------------------------------------------
    double Tinit, Tfinal, Tincrement; //Temperature related variables
    int Tsign; //The sign of Tincrement
    int Tsub;//No. subdivisions; Smallest interval=Tincrement/Tsub; 
    vector <double> mu_init; //Initial chem. pot
    vector <double> mu_final;//Final chem. pot
    vector <double> mu_increment; //Step size for increasing chem. pot
    int ndim; //No. of dimensions of system 
    int num_type;//No. of independent atomic types. E.g for binary:1 , for ternary: 2
    int num_sublat;//No. of sublattices in the system;
    char canonical;// Choice of canonical: c or grand canonical: g
    int nsize;// Total no. of cells in a sublattice. nsize=nrep[0]*nrep[1]*nrep[2]
    int natoms;// Total no. of atoms in system;
    vector <int> nrep; //Stores no. of cells in the x, y, z direction
    vector<double> init_conc;//Initial concentrations of atom species (canonical only)
    vector <int> factor;//Used for conversion from cartesian coordinate to ijk.  
    int randseed;// Seed for random number generator
    int write_freq;//Output data every write_freq passesP
    int equil_pass; //No. of passes for equilibration
    int numpass; //Total no. of passes for per simulation
    int restart;// 0: start from randonised lattice  1: Start from config file (inconf0) 2: Start from fixed composition
    char inconf0[30];//Input file for configuration
    int maxbody;//Maximum no. of interacting bodies
    bool correlation_option;//0 for none, 1 for yes
    int pair_option;//0 for none, 1 for with Cutoff, 2 for All
    double pair_cutoff;//max cell distance for pair correlations (for cutoff option)
    //Pot Pots; // Class Pot is where potential datas are stored
    bool user_def_option;//0 for none, 1 for user define option
    vector<Cor_def_param *> user_def_param_spa;//Parameters for user defined correlated site (spatialy averaged)
    vector<Cor_def_param *> user_def_param_fix;//Param for user defined corr. fixed sites
    int LRO_option;//0 for none, 1 for all sites
    ClusterTree clustree;
    int num_runs;//No. of runs per mu. If PHASE mode is on, num_run=2. Else num_run=1
    bool sinv_option;//If true, structural inversion will be performed to extract interactions
                     //else the interactions are read from a file. Default is false.
    //vector<char*> sinv_filenames;//names of inputfiles for structural inversion.
    vector< vector<Nei_pos> > neighborlist;//Relative positions of neighbors in cartesian(local) coordinates
    //vector< vector<double> > lat_vec;//Lattice vectors in cartesian_XYZ coordinates
                                     //E.g for FCC it can be (0.5 0.5 0),(0.5 0 0.5) (0 0.5 0.5)
    //vector< vector<double> > lat_vec_inv;//Inverse matrix of lat_vec
    //vector<double> lat_vec_mag2;//Magnitude of the lattice vectors
    //double lat_vec_scale;//Scaling factor for lattice vector
    //vector < vector<double> > sublat_pos; //Sublattice vector positions
    vector<Element> elem_index;//Element or chemical species to be associated with indices 0, 1, 2...num_type  
    Trans_vectors Lat_vec;//Lattice translation vectors
    string sinv_dir;//Directory where inlatvec, inclus, inneighbors are stored for
                    //struct inv. Default is current directory
    string stdata_dir;//Directory where structural data are stored                
    double fracnn2;//Fraction of square of nearest neighbor distance. This accuracy is used during
                  //struc inv. to check if relaxed structure has the given lattice
    string outlevel;//Level of output (low or high)		  
    vector<int> nclimit;//Limit for f_index for clusters of a give nbody. Defines the optimal C.E
    //double minCVscore;//CVscore for the optimal cluster
    
    //----------------------------------------------------------------
    
    //Various member functions----------------------------------------
    double eEnergy();//Func. that initialise lattice with atoms before MC
    double update_energy();//Func. that updates energy of lattice with new chem. pot.
    //void init_noccupancy0();//Func. that initialise lattice with type 0 atom species
    void create_noccupancy();//Func. that initialise the occupation of the lattice sites
    int monte_carlo(); //Func. doing grand canonical MC
    void metro_grand(double T, int pass);//Metropolis algor. per pass.
    void metro_canon(double T, int pass);
    //void trial ();
    double denergy(int ijk, int l, int noccu_new);//Calc. energy change in  system
    //Look-up table for neighbors of a site. The site is given by
    //ijkl index. The neighbors correspond to n, f ,d cluster.
    //int neighbor_table(int n, int f, int d, int ii, int jj, int kk, int ll, int nei_ijkl[][4] );
    void cal_conc(); //Func. which calculates atomic concentration  
    //void init();//Func. which initialises the lattice.
    int  read_config( fstream& confin );
    void print_config(fstream& outconfig); //Print configuration of a lattice.
    void print_output(fstream& out, double temperature);//Func. which prints to file output.txt
    void print_output2(fstream& out, double temperature);//Func. which prints to file output2.txt
    void print_output3(fstream& out, int pass);//Func. which prints to file output3.txt
    //int get_noccu(const int sublat, int site);//Returns the atomic type at given sublattice and site.
    //void put_noccu(int sublat, int site, int occupancy);//Put atomic type at given sublattice and site.
    //int degenerate(int nbody,int cluster);//Temporary. Returns num of degenerate clusters.
    void read_clus(); //Read in cluster information from clus.in
    void read_neighbor();//Read in neighbor info from neighbor.in and store in noccupancy[ijk].site[l].neighbor
    void read_Vin();//Read in potential for various interaction from pot.in
    double get_Vin(int l, int n, int f, int c_occu[]);//get Vin
    //int trial_move(int noccu);//Generate a random trial move during MC
    void ijk_to_cartesian(int ijk, int cartesian[]);
    //Convert from int. ijk representation of lattice sites to representation in cartesian co-ordinates
    int cartesian_to_ijk(int cartesian[]); //convert back to ijk representation
    void boundary(int cartesian[],int nei_cartesian[], int cartesian_r[]);//B.C check
    void create_corn();//Initialise neighbor list for correlations
    void correlate();//Accumulate correlation terms
    //void avg_correlate(bool option, int countpass);//average cor terms over pass with option to ave over degenerate sites
    void print_corr(fstream& out,int countpass, double T);//Prints SRO;
    int linear_index(const int num_body, const vector<int> &indices);
    /*The above func returns the linear index for the interaction between
      num_body cluster with num_type species. E.g for num_type=2, the 
      interactions are 11, 12, 21, 22 (type 0 species is excluded). The
      corresponding linear indices are then 0, 1, 2, 3 respectively.
    */
    void tensor_index(const int num_body, const int lindex, vector<int>& tensor);
    /*The above func returns the tensorial index for the interaction between
      num_body cluster with num_type species. E.g for num_type=2, the 
      interactions are 11, 12, 21, 22 (type 0 species is excluded). The
      corresponding linear indices are then 0, 1, 2, 3 respectively.
      The linear index(lindex) is stored converted to tensor form and stored
      in tensor.The nature of the func. is reciprocal to int linear_index(....)
    */
    //vector<int>* min_index(int num_pos, int num_type);
    /*The above function outputs the minimum number of indexes
      e.g in a case of 3 sites with 2 addition species, we have
      111,112,121,122,211,212,221,222. But knowing e.g 112==211,
      the min_index func gives 111,112,122,222
    */
    void cal_conc_LRO();//Func. which cal. atom concentration and terms realted to LRO
    void cal_conc_cor();//Func. which cal. atom concentration and correlation terms.
    void print_LRO(fstream& out,int countpass, double T);//Prints LRO;
    void print_LROc(fstream& out,int countpass, double T);//Prints LRO in config format
    bool read_input();//Reads simulation inputs from file 
    bool print_param();//Prints parameters of MC run to file
    void sinversion();//Performs structural inversion
    void select_clus(Inversion& Inv);//Select optimal cluster combination from the 'master'
                                     //correlation matrix in Inv
    //void read_lat_vec();//Input lattice vectors.
    void cal_corr_disorder(Inversion& Inv);//Calculates the correlation for disorder state
    int prep_Vin(Inversion&);//Allocate sizes of Vin vectors in class Ncluster  to facilitate structural inversion
                         //In addition it calculates the correlations of disordered states.
    void create_Vin(const vector<double>& J);//Transfer interactions to Vin vectors in class Ncluster and print to file.
    //-----------------------------------------------------------------
    
    //Constructors and destructors-------------------------------------
    Lattice ();
    ~Lattice ();
    //-----------------------------------------------------------------

    //Protected data members-------------------------------------------
  protected:
    vector <CellO> noccupancy;
    //Denotes occupation of lattice site ijk l. 
    //To access atom type at site: noccupancy[ijk].site[l].atomtype
    //To access site's neigbors' ijk: noccupancy[ijk].site[l].neighbor[n].ijk
    //To access site's neigbors'l: noccupancy[ijk].site[l].neighbor[n].l
    //Where: i j k, are coordinates in x y z dirs.resecptively. 
    //l denotes the sublattice index
    //ijk is an integer which represent the vector (i, j, k)
    
    Energy E_struct;//A struct which stores values related to Energy
    //vector<int*> neig_list;//Stores the neigbor list of simple cubic latttice
    Concentration C_struct;//A struct storing conctration related terms
    //vector<double> conc; //Stores the conc. of each atomic species.
    //vector<double> conc_avg;//Stores the average conc. of each atom type
    //vector<int>amount; //Stores the no. of atoms for each species.
    
    //-----------------------------------------------------------------
};  
#endif

//EOF
